<?php

namespace PlagScanApi;

require_once  __DIR__ . '/Base.php';
/**
 * Class Documents
 * @author Ingmar Szmais <iszmais@databay.de>
 * @since 20.08.19
 */
class Documents extends Base
{
    /**
     * @param int $docID
     * @return array|string
     */
    public function get(int $docID)
    {
        return $this->handler->request('documents/' . $docID, 'GET', $this->accessToken);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function getStatus(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/status', 'GET', $this->accessToken);
    }

    /**
     * @param string $fileData
     * @param bool $plain
     * @param int $userID
     * @return array|string
     */
    public function add(string $fileData, bool $plain, int $userID = 0)
    {
        if ($plain) {
            $data['textdata'] = $fileData;
        } else {
            $data['fileUpload'] = $fileData;
        }
        if($userID){
            $data['userID'] = $userID;
        }
        return $this->handler->request('documents', 'POST', $this->accessToken, $data);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function delete(int $docID)
    {
        return $this->handler->request('documents/' . $docID, 'DELETE', $this->accessToken);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function archive(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/archive', 'PUT', $this->accessToken);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function check(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/check', 'PUT', $this->accessToken);
    }

    /**
     * @param int $docID
     * @param int $mode
     * @param int $userID
     * @return array|string
     */
    public function retrieveReport(int $docID, int $mode = 0, int $userID = 0)
    {
        $data['mode'] = $mode;
        if ($userID) {
            $data['userID'] = $userID;
        }
        return $this->handler->request('documents/' . $docID . '/retrieve', 'GET', $this->accessToken);
    }

    /**
     * @param int $docID
     * @param string $format
     * @return array|string
     */
    public function retrieveAllSources(int $docID, string $format = 'old_html')
    {
        $data['format'] = $format;
        return $this->handler->request('documents/' . $docID . '/sources', 'GET', $this->accessToken, $data);
    }

    /**
     * @param int $docID
     * @param string $source
     * @param string $format
     * @return array|string
     */
    public function retrieveHighlightedSources(int $docID, string $source, string $format = 'old_html')
    {
        $data['source'] = $source;
        $data['format'] = $format;
        return $this->handler->request('documents/' . $docID . '/highlight', 'GET', $this->accessToken, $data);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function privatize(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/private', 'PUT', $this->accessToken);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function publish(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/private', 'DELETE', $this->accessToken);
    }

    /**
     * @param int $docID
     * @param array $config
     * @return array|string
     */
    public function share(int $docID, array $config = [])
    {
        return $this->handler->request('documents/' . $docID . '/share', 'PUT', $this->accessToken, $config);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function unshare(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/share', 'DELETE', $this->accessToken);
    }

    /**
     * @param int $docID
     * @return array|string
     */
    public function getShareLink(int $docID)
    {
        return $this->handler->request('documents/' . $docID . '/share', 'GET', $this->accessToken);
    }
}