<?php

namespace PlagScanApi;

require_once  __DIR__ . '/Base.php';
/**
 * Class Users
 * @author Ingmar Szmais <iszmais@databay.de>
 * @since 20.08.19
 */
class Users extends Base
{

    /**
     * @param int $start
     * @param int $limit
     * @return array|string
     */
    public function getAll(int $start = 0, int $limit = 0)
    {
        $data = [];
        if ($start) {
            $data['start'] = $start;
        }
        if ($limit) {
            $data['limit'] = $limit;
        }

        return $this->handler->request('users', 'GET', $this->accessToken, $data);
    }

    /**
     * Return user for the give Plagscan ID
     *
     * @param int $userID
     * @return array|string
     */
    public function get(int $userID)
    {
        return $this->handler->request('users/' . $userID, 'GET', $this->accessToken);
    }

    /**
     * @param string $username
     * @param string $firstname
     * @param string $lastname
     * @param string $email
     * @param int $group
     * @return array|string
     */
    public function add(string $username, string $firstname, string $lastname, string $email, int $group = 0)
    {
        $data = [
            'username' => $username,
            'firstname' => $firstname,
            'lastname' => $lastname,
            'email' => $email
        ];

        if ($group !== 0) {
            $data['deptID'] = $group;
        }

        return $this->handler->request('users', 'POST', $this->accessToken, $data);
    }

    /**
     * @param int $userID
     * @param array $changes
     * @return array|string
     */
    public function changeProperties(int $userID, array $changes)
    {
        return $this->handler->request('users/' . $userID, 'PATCH', $this->accessToken, $changes);
    }

    /**
     * @param int $userID
     * @return array|string
     */
    public function delete(int $userID)
    {
        return $this->handler->request('users/' . $userID, 'DELETE', $this->accessToken);
    }

    /**
     * @param int $userID
     * @param string $shared
     * @param int $start
     * @param int $limit
     * @param int $mode
     * @return array|string
     */
    public function getDocuments(int $userID, string $shared = 'withMe', int $start = 0, int $limit = 0, int $mode = 0)
    {
        $data['shared'] = $shared;
        if ($start) {
            $data['start'] = $start;
        }
        if ($limit) {
            $data['limit'] = $limit;
        }
        if ($mode) {
            $data['mode'] = $mode;
        }
        return $this->handler->request('users/' . $userID . '/documents', 'GET', $this->accessToken, $data);
    }

    public function archiveDocuments(int $userID) {
        return $this->handler->request('users/' . $userID . '/documents/archive', 'PUT', $this->accessToken);
    }

    /**
     * @param int $userID
     * @param int $start
     * @param int $limit
     * @return array|string
     */
    public function getRepositoryDocuments(int $userID, int $start = 0, int $limit = 0)
    {
        $data = [];
        if ($start) {
            $data['start'] = $start;
        }
        if ($limit) {
            $data['limit'] = $limit;
        }

        return $this->handler->request('users/' . $userID . '/repository/documents', 'GET', $this->accessToken, $data);
    }
}