<?php
/**
 * @category PlagScanExercise
 * @package PlagScan
 * @author Maximilian Becker <mbecker@databay.de>
 * @copyright 2019 Maximilian Becker / Databay AG
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link http://www.databay.de Databay AG
 */

// @codingStandardsIgnoreStart
require_once './Services/Component/classes/class.ilPluginConfigGUI.php';
require_once './Services/Form/classes/class.ilPropertyFormGUI.php';
require_once './Services/Form/classes/class.ilMultiSelectInputGUI.php';
// @codingStandardsIgnoreEnd

/**
 * Class ilCategoryCourseAssignmentsConfigGUI
 *
 * @ilCtrl_isCalledBy ilPlagScanExerciseConfigGUI: ilAdministrationGUI
 * @ilCtrl_Calls ilPlagScanExerciseConfigGUI: ilPropertyFormGUI
 */
class ilPlagScanExerciseConfigGUI extends ilPluginConfigGUI
{

    //<editor-fold desc="Fields">

    /** @var $tpl \HTML_Template_ITX|\ilTemplate */
    protected $tpl;

    /** @var $ilTabs ilTabsGUI */
    protected $ilTabs;

    /** @var $ilCtrl ilCtrl */
    protected $ilCtrl;

    /** @var $lng ilLanguage */
    protected $lng;

    /** @var ilSetting $ilSetting */
    protected $ilSetting;

    //</editor-fold>

    /**
     * ilPlagScanExerciseConfigGUI constructor.
     */
    public function __construct() {
        global $tpl, $ilTabs, $ilCtrl, $lng; // TODO: DIC

        $this->tpl = $tpl;
        $this->ilTabs = $ilTabs;
        $this->ilCtrl = $ilCtrl;
        $this->lng = $lng;
        $this->lng->loadLanguageModule('ui_uihk_psx');
        $this->ilSetting = new ilSetting('psx');
    }

    //<editor-fold desc="Plugin Boilerplate">

    /**
     * @return void
     * @throws \ilCtrlException
     *
     */
    public function executeCommand() {
        $cmd = $this->ilCtrl->getCmd();
        $next_class = $this->ilCtrl->getNextClass();

        if ($next_class == 'ilpropertyformgui') {
            $form = $this->getSettingsForm();
            $this->ilCtrl->forwardCommand($form);

        }
        parent::executeCommand();

    }

    /**
     * @param string $a_cmd Command to be executed.
     *
     * Defaults to 'settings', calling the general config screen.
     *
     * @return void
     */
    public function performCommand($a_cmd) {
        $this->buildTabs();

        switch (strtolower($a_cmd)) {
            case 'save':
                $this->ilTabs->activateTab('settings');
                $this->save();
                break;

            case 'configure':
            default:
                $this->ilTabs->activateTab('settings');
                $this->showSettingsForm();
                break;
        }
    }

    /**
     * @return void
     */
    protected function buildTabs() {
        $this->ilTabs->addTab(
            'settings',
            $this->lng->txt('settings'),
            $this->ilCtrl->getLinkTarget($this, 'configure')
        );
    }

    //</editor-fold>

    //<editor-fold desc="Configuration">

    /**
     * @return ilPropertyFormGUI
     */
    public function getSettingsForm() {
        $form = new ilPropertyFormGUI();
        $settings = new ilSetting('psx');

        $form->setFormAction($this->ilCtrl->getFormAction($this));
        $form->setTitle($this->lng->txt('ui_uihk_psx_settings'));
        $form->addCommandButton('save', $this->lng->txt('save'));

        $client_id = new ilTextInputGUI($this->lng->txt('ui_uihk_psx_set_clientid'), 'psx_clientid');
        $client_id->setValue($settings->get('psx_client_id', ''));
        $form->addItem($client_id);

        $api_key = new ilTextInputGUI($this->lng->txt('ui_uihk_psx_set_apikey'), 'psx_apikey');
        $api_key->setValue($settings->get('psx_apikey', ''));
        $form->addItem($api_key);

        $language_selector = new ilSelectInputGUI($this->lng->txt('ui_uihk_psx_set_lang'), 'psx_lang');
        $language_selector->setOptions(
            array(
                'english' => 'I am a poor little dropdown that wants meaningful options.'
            )
        );
        $language_selector->setValue($settings->get('psx_lang')); // DEFAULT?
        $form->addItem($language_selector);

        $email_policy = new ilSelectInputGUI($this->lng->txt('ui_uihk_psx_set_email'), 'psx_email');
        $email_policy->setOptions(
            array(
                'no_mails' => $this->lng->txt('ui_uihk_psx_set_emp_none'),
                'mail_all' => $this->lng->txt('ui_uihk_psx_set_emp_all'),
                'mail_red' => $this->lng->txt('ui_uihk_psx_set_emp_red'),
            )
        );
        $email_policy->setValue($settings->get('psx_email', 'no_mails'));
        $form->addItem($email_policy);

        $checks_field = new ilCheckboxGroupInputGUI($this->lng->txt('ui_uihk_psx_set_checks'), 'psx_checks');
        $checks_field->addOption(
          new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_chk_web'),'psx_chk_web')
        );
        $checks_field->addOption(
          new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_chk_my'),'psx_chk_my')
        );
        $checks_field->addOption(
          new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_chk_repo'),'psx_chk_repo')
        );
        $checks_field->addOption(
          new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_chk_orgrepo'),'psx_chk_orgrepo')
        );
        $checks_field->addOption(
          new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_chk_prev'),'psx_chk_prev')
        );
        $form->addItem($checks_field);
        if($settings->get('psx_chk_web', false))
        {
            $form->getItemByPostVar('psx_chk_web')->setChecked(true);
        }
        if($settings->get('psx_chk_my', false))
        {
            $form->getItemByPostVar('psx_chk_my')->setChecked(true);
        }
        if($settings->get('psx_chk_repo', false))
        {
            $form->getItemByPostVar('psx_chk_repo')->setChecked(true);
        }
        if($settings->get('psx_chk_orgrepo', false))
        {
            $form->getItemByPostVar('psx_chk_orgrepo')->setChecked(true);
        }
        if($settings->get('psx_chk_prev', false))
        {
            $form->getItemByPostVar('psx_chk_prev')->setChecked(true);
        }

        $student_refusal = new ilCheckboxInputGUI($this->lng->txt('ui_uihk_psx_set_refusal'), 'psx_refusal');
        $student_refusal->setChecked($settings->get('psx_refusal', false) ? true : false);
        $form->addItem($student_refusal);

        $student_disclosure = new ilTextAreaInputGUI($this->lng->txt('ui_uihk_psx_set_disclosure'), 'psx_disclosure');
        $student_disclosure->setRows(10);
        $student_disclosure->setValue($settings->get('psx_disclosure', ''));
        $form->addItem($student_disclosure);

        $storage_limit = new ilSelectInputGUI($this->lng->txt('ui_uihk_psx_set_storage'), 'psx_storage');
        $storage_limit->setOptions(
            array(
                '1_week' => $this->lng->txt('ui_uihk_psx_set_sto_week'),
                '3_months' => $this->lng->txt('ui_uihk_psx_set_sto_3months'),
                '6_months' => $this->lng->txt('ui_uihk_psx_set_sto_6months'),
                'never' => $this->lng->txt('ui_uihk_psx_set_sto_never'),
            )
        );
        $storage_limit->setValue($settings->get('psx_storage', '6_months'));
        $form->addItem($storage_limit);

        $sensitity = new ilSelectInputGUI($this->lng->txt('ui_uihk_psx_set_sensitivity'), 'psx_sensitivity');
        $sensitity->setOptions(
            array(
                'low' => $this->lng->txt('ui_uihk_psx_set_sens_low'),
                'medium' => $this->lng->txt('ui_uihk_psx_set_sens_medium'),
                'high' => $this->lng->txt('ui_uihk_psx_set_sens_high'),
            )
        );
        $sensitity->setValue($settings->get('psx_sensitivity', 'low'));
        $form->addItem($sensitity);

        // Associate uploads with -> not in first version

        $admin_mail = new ilEMailInputGUI($this->lng->txt('ui_uihk_psx_set_adminmail'), 'psx_adminmail');
        $admin_mail->setValue($settings->get('psx_adminmail', ''));
        $form->addItem($admin_mail);

        return $form;
    }

    /**
     * @return void
     */
    public function showSettingsForm() {
        $form = $this->getSettingsForm();
        $this->tpl->setContent($form->getHTML());
    }

    /**
     * @return void
     */
    public function save() {
        global $tpl, $ilCtrl;

        $pl = $this->getPluginObject();
        $form = $this->getSettingsForm();

        if ($form->checkInput()) {
            foreach ($_POST as $post_data_key => $post_data_value) {
                if ($post_data_key == 'cmd') {
                    continue;
                }

                if ($post_data_value . $post_data_key == '') {
                    continue;
                }

                if (is_array($post_data_value)) {
                    $post_data_value = implode(',', $post_data_value);
                }

                $this->ilSetting->set($post_data_key, $post_data_value);
            }
            ilUtil::sendSuccess($pl->txt('saving_invoked'), true);
            $ilCtrl->redirect($this, 'configure');
        } else {
            $form->setValuesByPost();
            $tpl->setContent($form->getHTML());
        }
    }

    //</editor-fold>

}